/*
 * Copyright Elasticsearch B.V. and/or licensed to Elasticsearch B.V. under one
 * or more contributor license agreements. Licensed under the Elastic License
 * 2.0; you may not use this file except in compliance with the Elastic License
 * 2.0.
 */

import React from 'react';
import type {
  AppMountParameters,
  CoreSetup,
  Plugin,
  PluginInitializerContext,
  CoreStart,
} from 'src/core/public';
import { i18n } from '@kbn/i18n';

import type { NavigationPublicPluginStart } from 'src/plugins/navigation/public';

import type {
  CustomIntegrationsStart,
  CustomIntegrationsSetup,
} from 'src/plugins/custom_integrations/public';

import type { SharePluginStart } from 'src/plugins/share/public';

import { once } from 'lodash';

import type { UsageCollectionSetup } from '../../../../src/plugins/usage_collection/public';

import { DEFAULT_APP_CATEGORIES, AppNavLinkStatus } from '../../../../src/core/public';

import type {
  DataPublicPluginSetup,
  DataPublicPluginStart,
} from '../../../../src/plugins/data/public';
import { FeatureCatalogueCategory } from '../../../../src/plugins/home/public';
import type { HomePublicPluginSetup } from '../../../../src/plugins/home/public';
import { Storage } from '../../../../src/plugins/kibana_utils/public';
import type { LicensingPluginSetup } from '../../licensing/public';
import type { CloudSetup } from '../../cloud/public';
import type { GlobalSearchPluginSetup } from '../../global_search/public';
import {
  PLUGIN_ID,
  INTEGRATIONS_PLUGIN_ID,
  setupRouteService,
  appRoutesService,
  calculateAuthz,
} from '../common';
import type { CheckPermissionsResponse, PostFleetSetupResponse, FleetAuthz } from '../common';

import type { FleetConfigType } from '../common/types';

import { CUSTOM_LOGS_INTEGRATION_NAME, INTEGRATIONS_BASE_PATH } from './constants';
import { licenseService } from './hooks';
import { setHttpClient } from './hooks/use_request';
import { createPackageSearchProvider } from './search_provider';
import { TutorialDirectoryHeaderLink, TutorialModuleNotice } from './components/home_integration';
import { createExtensionRegistrationCallback } from './services/ui_extensions';
import type { UIExtensionRegistrationCallback, UIExtensionsStorage } from './types';
import { LazyCustomLogsAssetsExtension } from './lazy_custom_logs_assets_extension';

export type { FleetConfigType } from '../common/types';

import { setCustomIntegrations } from './services/custom_integrations';

// We need to provide an object instead of void so that dependent plugins know when Fleet
// is disabled.
// eslint-disable-next-line @typescript-eslint/no-empty-interface
export interface FleetSetup {}

/**
 * Describes public Fleet plugin contract returned at the `start` stage.
 */
export interface FleetStart {
  /** Authorization for the current user */
  authz: Promise<FleetAuthz>;
  registerExtension: UIExtensionRegistrationCallback;
  isInitialized: () => Promise<true>;
}

export interface FleetSetupDeps {
  licensing: LicensingPluginSetup;
  data: DataPublicPluginSetup;
  home?: HomePublicPluginSetup;
  cloud?: CloudSetup;
  globalSearch?: GlobalSearchPluginSetup;
  customIntegrations: CustomIntegrationsSetup;
  usageCollection?: UsageCollectionSetup;
}

export interface FleetStartDeps {
  data: DataPublicPluginStart;
  navigation: NavigationPublicPluginStart;
  customIntegrations: CustomIntegrationsStart;
  share: SharePluginStart;
}

export interface FleetStartServices extends CoreStart, FleetStartDeps {
  storage: Storage;
  share: SharePluginStart;
  cloud?: CloudSetup;
  authz: FleetAuthz;
}

export class FleetPlugin implements Plugin<FleetSetup, FleetStart, FleetSetupDeps, FleetStartDeps> {
  private config: FleetConfigType;
  private kibanaVersion: string;
  private extensions: UIExtensionsStorage = {};
  private storage = new Storage(localStorage);

  constructor(private readonly initializerContext: PluginInitializerContext) {
    this.config = this.initializerContext.config.get<FleetConfigType>();
    this.kibanaVersion = initializerContext.env.packageInfo.version;
  }

  public setup(core: CoreSetup<FleetStartDeps, FleetStart>, deps: FleetSetupDeps) {
    const config = this.config;
    const kibanaVersion = this.kibanaVersion;
    const extensions = this.extensions;

    setCustomIntegrations(deps.customIntegrations);

    // TODO: this is a contract leak and an issue.  We shouldn't be setting a module-level
    // variable from plugin setup.  Refactor to an abstraction, if necessary.
    // Set up http client
    setHttpClient(core.http);

    // Set up license service
    licenseService.start(deps.licensing.license$);

    // Register Integrations app
    core.application.register({
      id: INTEGRATIONS_PLUGIN_ID,
      category: DEFAULT_APP_CATEGORIES.management,
      appRoute: '/app/integrations',
      title: i18n.translate('xpack.fleet.integrationsAppTitle', {
        defaultMessage: 'Integrations',
      }),
      order: 9019,
      euiIconType: 'logoElastic',
      mount: async (params: AppMountParameters) => {
        const [coreStartServices, startDepsServices, fleetStart] = await core.getStartServices();
        const startServices: FleetStartServices = {
          ...coreStartServices,
          ...startDepsServices,
          storage: this.storage,
          cloud: deps.cloud,
          authz: await fleetStart.authz,
        };
        const { renderApp, teardownIntegrations } = await import('./applications/integrations');

        const Tracker =
          deps.usageCollection?.components.ApplicationUsageTrackingProvider ?? React.Fragment;
        const unmount = renderApp(
          startServices,
          params,
          config,
          kibanaVersion,
          extensions,
          Tracker
        );

        return () => {
          unmount();
          teardownIntegrations(startServices);
        };
      },
    });

    // Register main Fleet app
    core.application.register({
      id: PLUGIN_ID,
      category: DEFAULT_APP_CATEGORIES.management,
      title: i18n.translate('xpack.fleet.appTitle', { defaultMessage: 'Fleet' }),
      order: 9020,
      euiIconType: 'logoElastic',
      appRoute: '/app/fleet',
      mount: async (params: AppMountParameters) => {
        const [coreStartServices, startDepsServices, fleetStart] = await core.getStartServices();
        const startServices: FleetStartServices = {
          ...coreStartServices,
          ...startDepsServices,
          storage: this.storage,
          cloud: deps.cloud,
          authz: await fleetStart.authz,
        };
        const { renderApp, teardownFleet } = await import('./applications/fleet');
        const unmount = renderApp(startServices, params, config, kibanaVersion, extensions);

        return () => {
          unmount();
          teardownFleet(startServices);
        };
      },
    });

    // BWC < 7.11 redirect /app/ingestManager to /app/fleet
    core.application.register({
      id: 'ingestManager',
      category: DEFAULT_APP_CATEGORIES.management,
      navLinkStatus: AppNavLinkStatus.hidden,
      title: i18n.translate('xpack.fleet.oldAppTitle', { defaultMessage: 'Ingest Manager' }),
      async mount(params: AppMountParameters) {
        const [coreStart] = await core.getStartServices();
        coreStart.application.navigateToApp('fleet', {
          path: params.history.location.hash,
        });
        return () => {};
      },
    });

    // Register components for home/add data integration
    if (deps.home) {
      deps.home.tutorials.registerDirectoryHeaderLink(PLUGIN_ID, TutorialDirectoryHeaderLink);
      deps.home.tutorials.registerModuleNotice(PLUGIN_ID, TutorialModuleNotice);

      deps.home.featureCatalogue.register({
        id: 'fleet',
        title: i18n.translate('xpack.fleet.featureCatalogueTitle', {
          defaultMessage: 'Add Elastic Agent integrations',
        }),
        description: i18n.translate('xpack.fleet.featureCatalogueDescription', {
          defaultMessage: 'Add and manage integrations with Elastic Agent',
        }),
        icon: 'indexManagementApp',
        showOnHomePage: true,
        path: INTEGRATIONS_BASE_PATH,
        category: FeatureCatalogueCategory.DATA,
        order: 510,
      });
    }

    if (deps.globalSearch) {
      deps.globalSearch.registerResultProvider(createPackageSearchProvider(core));
    }

    return {};
  }

  public start(core: CoreStart): FleetStart {
    const registerExtension = createExtensionRegistrationCallback(this.extensions);
    const getPermissions = once(() =>
      core.http.get<CheckPermissionsResponse>(appRoutesService.getCheckPermissionsPath())
    );

    registerExtension({
      package: CUSTOM_LOGS_INTEGRATION_NAME,
      view: 'package-detail-assets',
      Component: LazyCustomLogsAssetsExtension,
    });

    return {
      // Temporarily rely on superuser check to calculate authz. Once Kibana RBAC is in place for Fleet this should
      // switch to a sync calculation based on `core.application.capabilites` properties.
      authz: getPermissions()
        .catch((e) => {
          // eslint-disable-next-line no-console
          console.warn(`Could not load Fleet permissions due to error: ${e}`);
          return { success: false };
        })
        .then((permissionsResponse) => {
          if (permissionsResponse.success) {
            // If superuser, give access to everything
            return calculateAuthz({
              fleet: { all: true, setup: true },
              integrations: { all: true, read: true },
            });
          } else {
            // All other users only get access to read integrations if they have the read privilege
            const { capabilities } = core.application;
            return calculateAuthz({
              fleet: { all: false, setup: false },
              integrations: { all: false, read: capabilities.fleet.read as boolean },
            });
          }
        }),

      isInitialized: once(async () => {
        const permissionsResponse = await getPermissions();

        if (permissionsResponse?.success) {
          const { isInitialized } = await core.http.post<PostFleetSetupResponse>(
            setupRouteService.getSetupPath()
          );
          if (!isInitialized) {
            throw new Error('Unknown setup error');
          }

          return true;
        } else {
          throw new Error(permissionsResponse?.error || 'Unknown permissions error');
        }
      }),

      registerExtension,
    };
  }

  public stop() {}
}
